@@ -2,7 +2,7 @@
  * GTimer
  *
  * Copyright:
- *	(C) 1999 Craig Knudsen, cknudsen@radix.net
+ *	(C) 1999 Craig Knudsen, cknudsen@cknudsen.com
  *	See accompanying file "COPYING".
  * 
  *	This program is free software; you can redistribute it and/or
@@ -24,10 +24,10 @@
  *	Helps you keep track of time spent on different tasks.
  *
  * Author:
- *	Craig Knudsen, cknudsen@radix.net, http://www.radix.net/~cknudsen
+ *	Craig Knudsen, cknudsen@cknudsen.com, http://www.cknudsen.com
  *
  * Home Page:
- *	http://www.radix.net/~cknudsen/gtimer/
+ *	http://www.cknudsen.com/gtimer/
  *
  * History:
  *	19-May-1999	Created (based on an 1995 app I wrote)
@@ -58,6 +58,7 @@
 #include <fcntl.h>
 #include <time.h>
 
+#include "project.h"
 #include "task.h"
 #include "gtimer.h"
 
@@ -503,8 +504,10 @@ sockfd connection;
 /*
 ** Do a generic get request.
 */
-httpError httpGet ( connection, path, qs_names, qs_values, num, callback )
+httpError httpGet ( connection, virthost, path, qs_names, qs_values,
+  num, callback )
 sockfd connection;
+char *virthost;
 char *path;
 char *qs_names[];
 char *qs_values[];
@@ -543,10 +546,17 @@ void (*callback)(char *,int);
   strcpy ( temp2, " HTTP/1.0\r\n" );
   strcat ( temp2, "User-Agent: " );
   strcat ( temp2, user_agent() );
-  strcat ( temp2, "\r\n\r\n" );
+  strcat ( temp2, "\r\n" );
+  if ( virthost != NULL ) {
+    strcat ( temp2, "Host: " );
+    strcat ( temp2, virthost );
+    strcat ( temp2, "\r\n" );
+  }
+  strcat ( temp2, "\r\n" );
   command = (char *) realloc ( command, strlen ( command ) +
     strlen ( temp2 ) + 1 );
   strcat ( command, temp2 );
+
   httpEnqueueRequest ( connection, command, httpReadGet, NULL, callback );
 
   return ( HTTP_NO_ERROR );
@@ -616,10 +626,11 @@ httpRequest *request;
       /* complete header found (starts in alldata) */
       ptr = strtok ( alldata, "\r\n" );
       while ( ptr ) {
-        if ( strncmp ( ptr, "HTTP/1.0 200", 12 ) == 0 ) {
+        if ( strncmp ( ptr, "HTTP/1.0 200", 12 ) == 0 ||
+          strncmp ( ptr, "HTTP/1.1 200", 12 ) == 0 ) {
           /* ok status. ignore */
         }
-        else if ( strncmp ( ptr, "HTTP/1.0", 8 ) == 0 ) {
+        else if ( strncmp ( ptr, "HTTP/1.", 7 ) == 0 ) {
           /* some other status -- error */
           request->gen_callback ( ptr, strlen ( ptr ) );   /* indicates error */
           request->gen_callback ( NULL, 0 );        /* indicates error */
